import 'package:flutter/foundation.dart';

@immutable
class Result<T> {
  const Result._(this._result);

  const Result.success({T? data}) : this._(data);

  Result.failure(String message) : this._(_Failure(message));

  final Object? _result;

  bool get isSuccess => !isFailure;

  bool get isFailure => _result is _Failure;

  void ifSuccess(void action(T value)) {
    if (isSuccess) {
      action(_result as T);
    }
  }

  void ifFailure(void action(String message)) {
    if (isFailure) {
      final message = (_result as _Failure).message;
      action(message);
    }
  }

  @override
  String toString() => isSuccess ? 'Success($_result)' : _result.toString();

  @override
  int get hashCode => runtimeType.hashCode ^ _result.hashCode;

  @override
  bool operator ==(Object other) => other is Result && other._result == _result;
}

@immutable
class _Failure {
  const _Failure(this.message);

  final String message;
}
