import 'package:flutter/services.dart';
import 'package:flutter_bucketeer/bucketeer.dart';
import 'package:flutter_bucketeer/result.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  TestWidgetsFlutterBinding.ensureInitialized();

  const channel = MethodChannel('jp.bucketeer.plugin/flutter');

  setUp(() async {
    channel.setMockMethodCallHandler((methodCall) async {
      switch (methodCall.method) {
        case 'initialize':
        case 'start':
        case 'stop':
        case 'setUser':
        case 'track':
          return {'status': true};
        case 'getStringVariation':
          return {'status': true, 'response': 'datadata'};
        case 'getIntVariation':
          return {'status': true, 'response': 1234};
        case 'getDoubleVariation':
          return {'status': true, 'response': 55.2};
        case 'getBoolVariation':
          return {'status': true, 'response': true};
      }
    });
  });

  tearDown(() {
    channel.setMockMethodCallHandler(null);
  });

  test('Bucketeer Tests', () async {
    expectLater(
        Bucketeer.instance.initialize(
          apiKey: "apikeyapikeyapikeyapikeyapikeyapikeyapikey",
          endpoint: 'demo.bucketeer.jp',
          featureTag: 'Flutter',
          debugging: true,
          logSendingIntervalMillis: 3000,
          logSendingMaxBatchQueueCount: 3,
          pollingEvaluationIntervalMillis: 3000,
        ),
        completion(equals(Result.success())));

    expectLater(
        Bucketeer.instance.start(), completion(equals(Result.success())));

    expectLater(
        Bucketeer.instance.stop(), completion(equals(Result.success())));

    expectLater(Bucketeer.instance.getStringVariation('feature-id'),
        completion(equals(Result.success(data: 'datadata'))));

    expectLater(Bucketeer.instance.getIntVariation('feature-id'),
        completion(equals(Result.success(data: 1234))));

    expectLater(Bucketeer.instance.getDoubleVariation('feature-id'),
        completion(equals(Result.success(data: 55.2))));

    expectLater(Bucketeer.instance.getBoolVariation('feature-id'),
        completion(equals(Result.success(data: true))));

    expectLater(Bucketeer.instance.track('goal-id'),
        completion(equals(Result.success())));
  });
}
